/*
 This file is part of the digitalSTROM ds485-client library.

 Copyright (C) 2010 digitalSTROM.org, Schlieren, Switzerland

 digitalSTROM ds485-client library is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 digitalSTROM ds485-client library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with digitalSTROM ds485-client library. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DS485CLIENT_H_
#define DS485CLIENT_H_

#include <digitalSTROM/ds485-core.h>

typedef enum {
    DSMAPI_VERSION_1 = 1,
    DSMAPI_VERSION_2 = 2,
} dsmapi_version_t;

typedef struct
{
    dsid_t id;
    uint8_t busAddress;
    dsmapi_version_t apiVersion;
} dsmapi_device_t;

typedef enum
{
    CLIENT_SERIAL,
    CLIENT_SOCKET,
    CLIENT_UNIX
} ds485_conn_t;

typedef enum
{
    STATE_IDLE,
    STATE_WAITING,
    STATE_RECEIVED,
    STATE_CONNECTED,
} ds485_client_response_state_t;

typedef struct
{
    ds485_client_response_state_t state;
    ds485_container_t request;
    ds485_container_t response;
    pthread_mutex_t mutex;
    pthread_cond_t condition;
} ds485_client_syncsend_t;

typedef struct
{
    ds485_client_response_state_t state;
    uint8_t length;
    dsid_t *devices;
    pthread_mutex_t mutex;
    pthread_cond_t condition;
} ds485_client_sync_bus_query_t;

typedef struct {
    ds485_client_response_state_t state;
    pthread_mutex_t mutex;
    pthread_cond_t condition;
} ds485_client_open_query_t;

struct _ds485_client_s;

typedef struct
{
    ds485_conn_t conntype;
    int (*open)(struct _ds485_client_s *cl, const dsid_t *ownDSID);
    int (*close)(struct _ds485_client_s *cl);
    int (*send)(struct _ds485_client_s *cl, ds485_container_t *command);
    int (*get_bus_devices)(struct _ds485_client_s *cl, dsid_t *device_list, int device_list_size);
    int (*get_bus_state)(struct _ds485_client_s *cl, bus_state_t *state);
    int (*get_dsid)(struct _ds485_client_s *cl);
    client_callback_state_t stateCallback;
    void *stateCallbackData;
    client_callback_container_t responseCallback;
    void *responseCallbackData;
    client_callback_bus_change_t busChangeCallback;
    void *busChangeCallbackData;

} ds485_client_interface_t;

/*
 * Device specific data
 */
typedef struct _dS485Device {
    /* to be filled by the application */
    dsid_t dsid;                      /* Unique 96-bit serial number */
    bus_state_t busState;             /* The dS485 bus state */
    ds485_api_callback_bus_change_t busStateChangeCallback;
    struct _dS485Packet* txList;      /* packets scheduled to send by system layer */
    ds485_api_callback_received_t frameReceptionCallback;
    void* priv;                          /* system pointer, linking back to OS data */
    struct _dS485Device* next;
} dS485Device;

typedef struct
{
    ds485Handle_t serialHandle;
    char *dName;
    uint8_t rxfilterFlags;
    dsid_t ownDSID;
} ds485_client_serial_t;

typedef struct
{
    int sockfd;
    char *dProtocol;
    char *dName;
    char *dPort;
} ds485_client_socket_t;

typedef struct _ds485_client_s
{
    bus_state_t busState;
    int flags;

    char *connspec;

    ds485_client_interface_t connInterface;
    ds485_client_serial_t serialClient;
    ds485_client_socket_t socketClient;

    uint8_t active;
    pthread_t connThread;
    ds485_client_syncsend_t syncSend;
    ds485_client_sync_bus_query_t busQuery;
    ds485_client_open_query_t openQuery;
} ds485_client_t;


extern int ds485_serial_client_init(ds485_client_t *cl);
extern int ds485_serial_client_open(ds485_client_t *cl, const dsid_t *ownDSID);
extern int ds485_serial_client_close(ds485_client_t *cl);
extern int ds485_serial_client_send_command(ds485_client_t *cl, ds485_container_t *request);
extern int ds485_serial_client_query_devices(ds485_client_t *cl, dsid_t *device_list, int device_list_size);
extern int ds485_serial_client_query_state(ds485_client_t *cl, bus_state_t *state);
extern int ds485_serial_client_get_dsid(ds485_client_t *cl);

extern int ds485_socket_client_init(ds485_client_t *cl);
extern int ds485_socket_client_open(ds485_client_t *cl, const dsid_t *dsid);
extern int ds485_socket_client_close(ds485_client_t *cl);
extern int ds485_socket_client_send_command(ds485_client_t *cl, ds485_container_t *request);
extern int ds485_socket_client_get_bus_devices(ds485_client_t *cl, dsid_t *device_list, int device_list_size);
extern int ds485_socket_client_get_bus_state(ds485_client_t *cl, bus_state_t *state);
extern int ds485_socket_client_get_dsid(ds485_client_t *cl);

extern void ds485_cleanup(ds485_client_t *cl);

#endif /* DS485CLIENT_H_ */
